from LogManager import LogManager, FakeLogTarget, StopRule
from LogManager import ConditionalLogRule, UnconditionalLogRule
from LogManager import ConditionalSuppressRule, UnconditionalSuppressRule
from LogManager import PSUBJECTS
from FileTarget import FileTarget
from SyslogTarget import SyslogTarget
from ConsoleTarget import ConsoleTarget
from Persistence import Persistent
from Acquisition import Implicit
from AccessControl.Owned import Owned
from OFS.ObjectManager import ObjectManager
from OFS.SimpleItem import SimpleItem
from AccessControl.Role import RoleManager
from App.Management import Tabs
from Globals import HTMLFile
import Globals, re, os

constructELMForm = HTMLFile('constructEventLog', globals())

def constructELM(self, id, title, REQUEST=None):
    """ This doc string needs to be here.  :-\ """
    ob = EventLogManager(id, title)
    self._setObject(id, ob)
    if REQUEST is not None:
        return self.manage_main(self, REQUEST, update_menu=1)

class EventLogManager(Persistent, Implicit, SimpleItem, RoleManager,
                      ObjectManager, Owned):
    """ This doc string needs to be here. :-\ """
    __allow_access_to_unprotected_subobjects__ = 0

    __ac_permissions__ = (
        ('Use Event Logging Services',
         ('', 'isActivated', 'getCallableIds', 'getSubjects',
          'getRegisteredTargets', 'getRuleInfo',
          'getSelectedTargets', 'LOG', '__call__'),
         ('Anonymous','Manager')),
        ('Change Event Logging Configuration',
         ('addRule', 'changeRule', 'activate', 'deactivate',
          'manage_config', 'index_html', 'manage_UndoForm'),
         ('Manager',)),
        )

    meta_type = 'Event Log Manager'

    manage_options = (
        {'label':'Configuration','action':'manage_config'},
        {'label':'Contents','action':'manage_main'},
        {'label':'Security','action':'manage_access'},
        {'label':'Ownership','action':'manage_owner'},
        {'label':'Undo','action':'manage_UndoForm'},
        )

    manage_config = HTMLFile('manageEventLog', globals())

    index_html = manage_config

    def __init__(self, id, title):
        self.id = id
        self.title = title
        self.LM = LogManager(DEFAULT_TARGETS)
        self.addRule('unconditional log', 'log everything',
                     DEFAULT_RULE_TARGETS)
        
    def isActivated(self):
        """ """
        return self.LM.isActivated()

    def activate(self, REQUEST=None):
        """ """
        self.LM.activate()
        self._p_changed = 1
        if REQUEST is not None:
            REQUEST.RESPONSE.redirect(REQUEST['URL1'])
            
    def deactivate(self, REQUEST=None):
        """ """
        self.LM.deactivate()
        self._p_changed = 1
        if REQUEST is not None:
            REQUEST.RESPONSE.redirect(REQUEST['URL1'])
            
        
    def getSubjects(self):
        """ """
        return tuple(PSUBJECTS)

    def getRegisteredTargets(self):
        """ """
        return tuple(self.LM.getRegisteredTargetNames())
    
    def getRules(self):
        """ returns num, isActivated, name, subject, predicate, targets """
        return self.LM.getRules()

    def getRuleTypes(self):
        """ """
        return RULE_TYPES.keys()

    def addRule(self, rule_type, rule_name, target_names=[], REQUEST=None):
        """ """
        rule_class = RULE_TYPES[rule_type]
        self.LM.appendRule(rule_class(rule_name, DEFAULT_SUBJECT,
                                      DEFAULT_PREDICATE, target_names))
        self._p_changed = 1
        if REQUEST is not None:
            REQUEST.RESPONSE.redirect(REQUEST['HTTP_REFERER'])

    def removeRule(self, index, REQUEST=None):
        """ """
        self.LM.popRule(index)
        self._p_changed = 1
        if REQUEST is not None:
            REQUEST.RESPONSE.redirect(REQUEST['URL1'])
        
    def makeRuleFirst(self, index, REQUEST=None):
        """ """
        rule = self.LM.popRule(index)
        self.LM.insertRule(0, rule)
        self._p_changed = 1
        if REQUEST is not None:
            REQUEST.RESPONSE.redirect(REQUEST['URL1'])
            

    def changeRule(self, index, name, subject, predicate_type,
                   predicate, targets=[], active=0, REQUEST=None):
        """ """
        rule = self.LM.popRule(index)
        try:
            c = RULE_TYPES[rule.getRuleType()]
            if predicate_type == 're':
                try:
                    predicate = re.compile(predicate)
                except:
                    raise "Bad regular expression in rule 'name',  %s" % name
            if predicate_type == 'callable':
                ob = getattr(self, 'aq_base', self) # do not acquire
                predicate = getattr(ob, predicate)
            if predicate_type == 'expr':
                predicate = str(predicate)
            newrule = c(name, subject, predicate, targets)
            if not active:
                newrule.deactivate()
            self.LM.insertRule(index, newrule)
            self._p_changed = 1
        except:
            self.LM.insertRule(index, rule)
            raise
        if REQUEST is not None:
            REQUEST.RESPONSE.redirect(REQUEST['HTTP_REFERER'])

    def getSelectedTargets(self, rule_name):
        l = []
        rule = self.LM.getRuleByName(rule_name)
        rule_targets = rule.getRuleTargets()
        for x in self.getRegisteredTargets():
            if x in rule_targets:
                l.append((x, 1))
            else:
                l.append((x,0))
        return l

    def LOG(self, subsystem, severity, summary, detail='', error=None,
            reraise=None):
        """ """
        if self.isActivated():
            self.LM.LOG(subsystem, severity, summary, detail, error, reraise)
        if reraise:
            raise
        
    __call__=LOG

class ELMBaseRule(Persistent, Implicit, SimpleItem):
    """ """
    __allow_access_to_unprotected_subobjects__ = 1

    __ac_permissions__ = (
        ('Use Event Logging Services',
         ('isActivated', 'getRuleTargetNames', 'getName', 'getRuleType',
          'getSubject', 'getPredicateType', 'getPredicateAsString'),
         ('Anonymous', 'Manager')),
        ('Change Event Logging Configuration',
         ('activate', 'deactivate', 'addRuleTarget', 'removeRuleTarget'),
         ('Manager',)),
        )

class ELMStopRule(ELMBaseRule, StopRule):
    """ rule superclass """
    meta_type = "Stop Rule"
    
class ELMConditionalLogRule(ELMBaseRule, ConditionalLogRule):
    """ rule superclass """
    meta_type = "Conditional Log Rule"
    
class ELMUnconditionalLogRule(ELMBaseRule, UnconditionalLogRule):
    """ rule superclass """
    meta_type = "Unconditional Log Rule"
    
class ELMConditionalSuppressRule(ELMBaseRule, ConditionalSuppressRule):
    """ rule superclass """
    meta_type = "Conditional Suppress Rule"
    
class ELMUnconditionalSuppressRule(ELMBaseRule, UnconditionalSuppressRule):
    """ rule superclass """
    meta_type = "Unconditional Suppress Rule"

RULE_TYPES = {
    'stop':ELMStopRule,
    'conditional log':ELMConditionalLogRule,
    'unconditional log':ELMUnconditionalLogRule,
    'conditional suppress':ELMConditionalSuppressRule,
    'unconditional suppress':ELMUnconditionalSuppressRule
    }

try:
    DEFAULT_TARGETS=_log_targets
    # this comes from __main__.__builtins__ (see __init__.py)
except:
    DEFAULT_TARGETS={}

DEFAULT_RULE_TARGETS = DEFAULT_TARGETS.keys()
DEFAULT_SUBJECT = 'subsystem'
DEFAULT_PREDICATE = ''

for klass in (EventLogManager, ELMStopRule, ELMUnconditionalLogRule,
              ELMConditionalLogRule, ELMUnconditionalSuppressRule,
              ELMConditionalSuppressRule):
    Globals.default__class_init__(klass)

