##############################################################################
# 
# Zope Public License (ZPL) Version 1.0
# -------------------------------------
# 
# Copyright (c) Digital Creations.  All rights reserved.
# 
# This license has been certified as Open Source(tm).
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
# 
# 1. Redistributions in source code must retain the above copyright
#    notice, this list of conditions, and the following disclaimer.
# 
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions, and the following disclaimer in
#    the documentation and/or other materials provided with the
#    distribution.
# 
# 3. Digital Creations requests that attribution be given to Zope
#    in any manner possible. Zope includes a "Powered by Zope"
#    button that is installed by default. While it is not a license
#    violation to remove this button, it is requested that the
#    attribution remain. A significant investment has been put
#    into Zope, and this effort will continue if the Zope community
#    continues to grow. This is one way to assure that growth.
# 
# 4. All advertising materials and documentation mentioning
#    features derived from or use of this software must display
#    the following acknowledgement:
# 
#      "This product includes software developed by Digital Creations
#      for use in the Z Object Publishing Environment
#      (http://www.zope.org/)."
# 
#    In the event that the product being advertised includes an
#    intact Zope distribution (with copyright and license included)
#    then this clause is waived.
# 
# 5. Names associated with Zope or Digital Creations must not be used to
#    endorse or promote products derived from this software without
#    prior written permission from Digital Creations.
# 
# 6. Modified redistributions of any form whatsoever must retain
#    the following acknowledgment:
# 
#      "This product includes software developed by Digital Creations
#      for use in the Z Object Publishing Environment
#      (http://www.zope.org/)."
# 
#    Intact (re-)distributions of any official Zope release do not
#    require an external acknowledgement.
# 
# 7. Modifications are encouraged but must be packaged separately as
#    patches to official Zope releases.  Distributions that do not
#    clearly separate the patches from the original work must be clearly
#    labeled as unofficial distributions.  Modifications which do not
#    carry the name Zope may be packaged in any form, as long as they
#    conform to all of the clauses above.
# 
# 
# Disclaimer
# 
#   THIS SOFTWARE IS PROVIDED BY DIGITAL CREATIONS ``AS IS'' AND ANY
#   EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
#   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
#   PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL DIGITAL CREATIONS OR ITS
#   CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
#   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
#   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
#   USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
#   ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
#   OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
#   OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
#   SUCH DAMAGE.
# 
# 
# This software consists of contributions made by Digital Creations and
# many individuals on behalf of Digital Creations.  Specific
# attributions are listed in the accompanying credits file.
# 
##############################################################################
import time, sys, string, os
import threading

lock = threading.Lock()

#from LogManagerInterfaces import LogManagerTargetInterface

TZ = time.timezone/3600

if abs(TZ) == TZ:
    TZS = "+%2.2d:00" % TZ
else:
    TZS = "-%2.2d:00" % TZ

del TZ

format_exception_only=None

def format_exception(etype,value,tb,limit=None,delimiter='\n',header='',
                     trailer='',linebegin='    ',join=string.join):
    global format_exception_only
    if format_exception_only is None:
        import traceback
        format_exception_only=traceback.format_exception_only
        
    result=["%s%s" % (linebegin, 'Traceback (innermost last):')]
    rappend= result.append
    if header: result.insert(0,'%s%s' % (linebegin, header))
    if limit is None:
        if hasattr(sys, 'tracebacklimit'):
            limit = sys.tracebacklimit
    n = 0
    while tb is not None and (limit is None or n < limit):
        f = tb.tb_frame
        lineno = tb.tb_lineno
        co = f.f_code
        filename = co.co_filename
        name = co.co_name
        locals=f.f_locals
        rappend(join(linebegin,'  File %s, line %d, in %s'
                     % (filename,lineno,name)))
        try: rappend(join(linebegin,'    (Object: %s)' %
                     locals[co.co_varnames[0]].__name__))
        except: pass
        try: rappend(join(linebegin,'    (Info: %s)' %
                     str(locals['__traceback_info__'])))
        except: pass
        tb = tb.tb_next
        n = n+1
    rappend(join(linebegin,join(format_exception_only(etype, value),' ')))
    if trailer: rappend(join(linebegin,trailer))
    
    return join(result, delimiter)

def log_time():
    """
    Return a simple time string without spaces suitable for logging
    (ISO8601 format - see http://www.w3.org/TR/NOTE-datetime)
    """
    t="%4.4d-%2.2d-%2.2dT%2.2d:%2.2d:%2.2d" % time.localtime(time.time())[:6]
    return "%s%s" % (t, TZS)

def severity_string(severity, mapping={
    -300: 'TRACE',
    -200: 'DEBUG',
    -100: 'BLATHER',
       0: 'INFO',       
     100: 'PROBLEM', 
     200: 'ERROR',    
     300: 'PANIC', 
    }):
    """Convert a severity code to a string
    """
    s=int(severity)
    if mapping.has_key(s): s=mapping[s]
    else: s=''
    return "%s(%s)" % (s, severity)

def get_detail(detail):
    if detail:
        return "%s\n" % detail
    return ''

class FileTarget:
    """ a file target """

    #__implements__ = LogManagerTargetInterface

    def __init__(self, f):
        self.dest=open(f,'a')

    def __call__(self, subsystem, severity, summary, detail='',error=None):
        lock.acquire()
        try:
            dest = self.dest
            dest.write(
                "%s %s %s %s\n"
                %
                (log_time(),
                 severity_string(severity),
                 subsystem,
                 summary,
                 )
                )
        
            if detail:
                dest.write("    %s\n" % detail)

            if error:
                try:
                    dest.write("    %s" % format_exception(
                        error[0], error[1], error[2],
                        trailer='\n', limit=100))
                except:
                    dest.write("%s: %s\n" % error[:2])
        
            dest.flush()
        finally:
            lock.release()






