from syslog import syslog_client, LOG_ALERT, LOG_ERR, LOG_WARNING, LOG_NOTICE
from syslog import LOG_INFO, LOG_DEBUG
import threading

lock = threading.Lock()

#from LogManagerInterfaces import LogManagerTargetInterface
import os, string

# disallowed characters for passing to syslogd
# return string that only has ascii 32 - 127 values in it
disallowed = string.join(map(chr, range(0x0, 0x20) + range(0x7F, 0xff)),'')
mt = string.maketrans("", "") # dummy translation table for string.translate

pid_str='[%s]: ' % os.getpid()

class SyslogTarget:
    """ a character-filtering Syslog Target """

    #__implements__ = LogManagerTargetInterface

    def __init__(self, addr=None, port=None):
        self.addr = addr
        self.port = port
        if addr is not None and port is not None:
            self.client = syslog_client((addr, int(port)))
        else:
            self.client = syslog_client()
        
    def __call__(self, subsystem, severity, summary, detail='',
                 error=None, mt=mt, disallowed=disallowed,
                 translate=string.translate, join=string.join):
        lock.acquire()
        try:
            if severity >= 0:
                if severity >= 200:
                    if severity >= 300:
                        severity=LOG_ALERT
                    else:
                        severity=LOG_ERR
                else:
                    if severity >= 100:
                        severity=LOG_WARNING
                    else:
                        severity=LOG_NOTICE
            else:
                if severity >= -100:
                    severity=LOG_INFO
                else:
                    severity=LOG_DEBUG
            if detail:
                msg = translate(join((summary, detail), ' - '), mt,
                                disallowed)
            else:
                msg = translate(summary, mt, disallowed)

            self.client.log(join((subsystem,pid_str,msg), ''),
                            priority=severity)
        finally:
            lock.release()


