import os, sys, re
program = sys.argv[0]
here = os.path.join(os.getcwd(), os.path.split(program)[0])
sys.path.insert(0, '../..')
from unittest import TestCase, TestSuite, TextTestRunner
import LogManager
clr = LogManager.ConditionalLogRule
ulr = LogManager.UnconditionalLogRule
csr = LogManager.ConditionalSuppressRule
usr = LogManager.UnconditionalSuppressRule
sr = LogManager.StopRule
# this module is for use with PyUnit (pyunit.sourceforge.net)

class testBase(TestCase):
    def setUp(self):
        self.logmgr = LogManager.LogManager()
        global clr, ulr, csr, usr, sr
        
    def tearDown(self):
        self.logmgr = None

class testLogManager(testBase):
    def onByDefault(self):
        assert self.logmgr.isActivated() == 1, ''

    def turnOn(self):
        self.logmgr.activate()
        assert self.logmgr.isActivated() == 1, ''

    def turnOff(self):
        self.logmgr.deactivate()
        assert self.logmgr.isActivated() == 0, ''

    def turnOffAndBackOn(self):
        self.logmgr.deactivate()
        self.logmgr.activate()
        assert self.logmgr.isActivated() == 1, ''

    def appendRule(self):
        r = sr('name1', 'subsystem', '> 1' , [])
        self.logmgr.appendRule(r)
        l = self.logmgr.getRules()
        assert len(l) == 1, ''
        
    def popRule(self):
        r = sr('name1','subsystem','==1',[])
        self.logmgr.appendRule(r)
        self.logmgr.popRule(0)
        assert len(self.logmgr.getRules()) == 0, ''

    def insertRule(self):
        r1 = sr('name1','subsystem','==1',[])
        r2 = sr('name2','severity','==1',[])
        r3 = sr('name3','detail','==1',[])
        for x in (r1, r2, r3):
            self.logmgr.appendRule(x)
        rx = sr('name4','summary','==1',[])
        self.logmgr.insertRule(0, rx)
        assert self.logmgr.getRules()[0] == rx, ''

    def ruleOrder(self):
        r1 = sr('name1','subsystem','==1',[])
        r2 = sr('name2','severity','==1',[])
        r3 = sr('name3','detail','==1',[])
        for x in (r1, r2, r3):
            self.logmgr.appendRule(x)
        rules = self.logmgr.getRules()
        assert len(rules) == 3, ''
        i = 0
        for x in (r1, r2, r3):
            assert rules[i] == x, ''
            i = i + 1

    def expressionIsWhatIThinkItIs(self):
        rule = clr('rule1','subsystem',"== 'ZODB'", [])
        tup = ('ZCatalog',100,'error','error', (None,None,None))
        expr = rule._getExpression(tup)
        assert expr == "'ZCatalog' == 'ZODB'", expr
        
    def stopRuleWorksOnMatch(self):
        dests = ['fake1', 'fake2']
        rule = sr('rule1','severity','== 100',dests)
        msg = ('ZODB',100,'error','error', (None,None,None))
        result = rule.match(msg, dests)
        assert result == -1, result
        assert dests == ['fake1','fake2'], dests

    def logRuleWorksOnMatch(self):
        dests = ['fake1','fake2']
        rule = clr('rule1','summary','=="error"',dests)
        msg = ('ZODB',100,'error','error', (None,None,None))
        result = rule.match(msg, dests)
        assert result == 1, result
        assert dests == ['fake1','fake2'], dests
        
    def suppressRuleWorksOnMatch(self):
        dests = ['fake1','fake2']
        targets = ['fake1','fake2']
        rule=csr('rule1','detail','=="error"',targets)
        msg = ('ZODB',100,'error','error', (None,None,None))
        result = rule.match(msg, dests)
        assert result == 1, result
        assert dests == [], dests

    def stopRuleWorksOnMiss(self):
        dests = ['fake1','fake2']
        rule = sr('rule1','error','==(None,None,1)',dests)
        msg = ('ZCatalog',100,'error','error', (None,None,None))
        result = rule.match(msg, dests)
        assert result == 0, result
        assert dests == ['fake1','fake2'], dests

    def logRuleWorksOnMiss(self):
        dests = ['fake1','fake2']
        rule = clr('rule1','subsystem','=="ZODB"',dests)
        msg = ('ZCatalog',100,'error','error', (None,None,None))
        result = rule.match(msg, dests)
        assert result == 0, ''
        assert dests == ['fake1','fake2'], dests

    def suppressRuleWorksOnMiss(self):
        dests = ['fake1','fake2']
        rule = csr('rule1','subsystem','=="ZODB"',dests)
        msg = ('ZCatalog',100,'error','error', (None,None,None))
        result = rule.match(msg, dests)
        assert result == 0, ''
        assert dests == ['fake1','fake2'], dests

    def checkCallablePredicate(self):
        def callable(val):
            if val == 'ZCatalog': return 1
        dests = ['fake1','fake2']
        rule = clr('rule1','subsystem',callable,dests)
        msg = ('ZCatalog',100,'error','error', (None,None,None))
        result = rule.match(msg, dests)
        assert result == 1, result
        assert dests == ['fake1','fake2'], dests

    def checkRegexPredicate(self):
        regex = re.compile('ZCatalog')
        dests = ['fake1','fake2']
        rule = clr('rule1','subsystem',regex,dests)
        msg = ('ZCatalog',100,'error','error', (None,None,None))
        result = rule.match(msg, dests)
        assert result == 1, result
        assert dests == ['fake1','fake2'], dests

    def getRuleTargets(self):
        r1 = sr('name1','subsystem','==1',[1,2])
        r2 = sr('name2','severity','==1',[3,4])
        r3 = sr('name3','detail','==1',[1,5])
        r4 = sr('name4','summary','==1',[4,5])
        l = self.logmgr
        for x in (r1, r2, r3, r4):
            l.appendRule(x)

        assert l.getRuleTargets() == [1,2,3,4,5], targets

    def getRegisteredTargets(self):
        t1 = LogManager.FakeLogTarget()
        t2 = LogManager.FakeLogTarget()
        t3 = LogManager.FakeLogTarget()
        t4 = LogManager.FakeLogTarget()
        l = LogManager.LogManager()
        dict = {'t1':t1, 't2':t2, 't3':t3, 't4':t4}
        names = self.logmgr.getRegisteredTargetNames()
        for x in names:
            l.removeRegisteredTarget(x)
        for x in dict.keys():
            l.addRegisteredTarget(x, dict[x])
        assert l.getRegisteredTargetNames()==dict.keys(),l.getRegisteredTargetNames()
        for x in dict.keys():
            l.removeRegisteredTarget(x)
        assert l.getRegisteredTargetNames() == [], l.getRegisteredTargetNames()
        self.logmgr.addRegisteredTarget('fake', t1)
        try:
            self.logmgr.addRegisteredTarget('fake', t1)
        except LogManager.LogManagerException:
            assert 1 == 1, 'ok'
        

    def testLOG(self):
        import re
        regex = re.compile("Fudgeling")
        target1 = LogManager.FakeLogTarget()
        target2 = LogManager.FakeLogTarget()
        target3 = LogManager.FakeLogTarget()
        target4 = LogManager.FakeLogTarget()
        dict = {'target1':target1, 'target2':target2, 'target3':target3,
                'target4':target4}
        for x in dict.keys():
            self.logmgr.addRegisteredTarget(x, dict[x])
        r1=clr('name1','subsystem','=="ZCatalog"',['target1'])
        r2=csr('name2','severity','<100', ['target1'])
        r3=clr('name3','subsystem',regex,['target4'])
        r4=clr('name4','subsystem','=="ZODB"',['target2'])
        r5=csr('name5','severity','<100', ['target2'])
        r6=ulr('name6', '','',['target3'])
        LOG = self.logmgr.LOG
        for x in (r1, r2, r3, r4, r5, r6):
            self.logmgr.appendRule(x)
        for x in range(0,100):
            LOG('ZCatalog',101,str(x)) # should go to t1, t3
        for x in range(0,100):
            LOG('ZODB',101,str(x)) # should go to t2, t3
        for x in range(0,100):
            LOG('ZCatalog',1,str(x)) # should go to t3
        for x in range(0,100):
            LOG('Foobar',1,str(x)) # should go to t3
        for x in range(0,100):
            LOG('Fudgeling', 0,str(x)) # should go to t4, t3
        t1 = target1.history
        t2 = target2.history
        t3 = target3.history
        t4 = target4.history
        assert len(t1) == 100, (len(t1), t1[0], t1[-1])
        assert len(t2) == 100, (len(t2), t2[0], t2[-1])
        assert len(t3) == 500, (len(t3), t3[0], t3[-1])
        assert len(t4) == 100, (len(t4), t4[0], t4[-1])
        
    def testInterfaces(self):
        target = LogManager.FakeLogTarget()
        r1 = clr('name1','subsystem','=="ZCatalog"',['target'])
        r2 = sr('name1','subsystem','=="ZCatalog"',['target'])
        r3 = csr('name1','subsystem','=="ZCatalog"',['target'])
        lmi = LogManager.LogManagerInterface
        ti = LogManager.LogManagerTargetInterface
        ri = LogManager.LogManagerRuleInterface
        assert ti.implementedBy(target), ''
        assert lmi.implementedBy(self.logmgr)
        for x in (r1, r2, r3):
            assert ri.implementedBy(x)

    def testStupidFileLogger(self):
        from ZLogger.stupidFileLogger import stupidFileLogger
        fl = stupidFileLogger()
        self.logmgr.addRegisteredTarget('fl', fl)
        r1 = ulr('rule', '', '', ['fl']) 
        self.logmgr.appendRule(r1)
        self.logmgr.LOG('','','')
        
class ConcreteSingleTestCase(TestCase):
    def setUp(self):
        pass

    def tearDown(self):
        pass

    def runTest(self):
        assert 1 == 1, 'one doesnt equal one'
        
if __name__ == '__main__':
    tests = (
        testLogManager('onByDefault'),
        testLogManager('turnOff'),
        testLogManager('turnOn'),
        testLogManager('turnOffAndBackOn'),
        testLogManager('appendRule'),
        testLogManager('popRule'),
        testLogManager('insertRule'),
        testLogManager('ruleOrder'),
        testLogManager('expressionIsWhatIThinkItIs'),
        testLogManager('stopRuleWorksOnMatch'),
        testLogManager('logRuleWorksOnMatch'),
        testLogManager('suppressRuleWorksOnMatch'),
        testLogManager('stopRuleWorksOnMiss'),
        testLogManager('logRuleWorksOnMiss'),
        testLogManager('suppressRuleWorksOnMiss'),
        testLogManager('checkCallablePredicate'),
        testLogManager('checkRegexPredicate'),
        testLogManager('getRuleTargets'),
        testLogManager('getRegisteredTargets'),
        testLogManager('testLOG'),
        testLogManager('testInterfaces'),
        testLogManager('testStupidFileLogger')
        )
    testsuite = TestSuite()
    for x in tests:
        testsuite.addTest(x)
    runner = TextTestRunner()
    runner.run(testsuite)

